Imports Microsoft.SqlServer.Server
Imports System.Collections
Imports System.Data.SqlTypes

' Represents a table row in memory
Public Class TableRow
    Public N As Integer
    Public Rate As Double
    Sub New(ByVal N As Integer, ByVal Rate As Double)
        Me.N = N
        Me.Rate = Rate
    End Sub
End Class

' Main class that implements an IEnumerator
Public Class TblLoader
    Implements IEnumerator

    ' Current values of the IEnumerator
    Private CurrentIndex As Integer = 0
    Private CurrentFactor As Double = 1
    ' Stored constructor arguments
    Private NumPeriods As Integer
    Private PercentInterest As Double

    ' Constructor with the table's arguments
    Sub New(ByVal NumPeriods As Integer, ByVal PercentInterest As Double)
        ' Store as class variables
        Me.NumPeriods = NumPeriods
        Me.PercentInterest = PercentInterest
    End Sub

    ' Returns the current row
    Public ReadOnly Property Current() As Object _
        Implements IEnumerator.Current
        Get
            Return New TableRow(CurrentIndex, CurrentFactor)
        End Get
    End Property

    ' Get the next element in the enumeration
    Public Function MoveNext() As Boolean _
        Implements IEnumerator.MoveNext
        If CurrentIndex < NumPeriods Then
            ' Calculate next value
            CurrentIndex += 1
            CurrentFactor *= (1 + PercentInterest / 100)
            Return True
        Else
            Return False
        End If
    End Function

    ' Reset the enumeration
    Public Overridable Sub Reset() _
        Implements IEnumerator.Reset
        CurrentIndex = 0
        CurrentFactor = 1
    End Sub
End Class

Public Class Interest
    ' Main function
    <SqlFunction(FillRowMethodName:="FillRow", _
        TableDefinition:="N int not null, Rate float not null")> _
    Public Shared Function AppreciationTbl(ByVal NumPeriods As Integer, _
        ByVal PercentInterest As Double) As IEnumerator
        Return New TblLoader(NumPeriods, PercentInterest)
    End Function

    ' Function called to retrieve each row
    Public Shared Sub FillRow(ByVal Obj As Object, ByRef N As SqlInt32, _
        ByRef Rate As SqlDouble)
        ' Check if the object is valid
        If Not (Obj Is Nothing) Then
            ' Retrieve the values
            Dim R As TableRow = CType(Obj, TableRow)
            N = R.N
            Rate = R.Rate
        Else
            ' Invalid object
            N = 0
            Rate = 1
        End If
    End Sub

End Class
